/***********************************************************************
 * $Id:: s1l_burner.c 8211 2011-10-05 17:00:11Z usb10132              $
 *
 * Project: SLC burner for stage 1 application
 *
 * Description:
 *     This version programs the kickstart or stage 1 application using
 *     the SLC with large block support.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

#include "lpc_types.h"
#include "lpc_irq_fiq.h"
#include "lpc32xx_slcnand_driver.h"
#include "lpc_arm922t_cp15_driver.h"
#include "lpc32xx_intc_driver.h"
#include "misc_config.h"
#include "common_funcs.h"

/* Prototype for external IRQ handler */
void lpc32xx_irq_handler(void);

/* NAND read and write buffers */
static DMA_BUFFER UNS_8 rdbuff[4096];

static NAND_GEOM_T nandgeom;
static INT_32 nand;
extern void * memset(void *a, int b, int sz);
/***********************************************************************
 *
 * Function: isblksame
 *
 * Purpose: Verify 2 blocks are the same
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     buff1 : Pointer to buffer 1 to verify
 *     buff2 : Pointer to buffer 2 to verify
 *
 * Outputs: None
 *
 * Returns: -1 on pass, or failed index
 *
 * Notes: None
 *
 **********************************************************************/
INT_32 isblksame(UNS_8 *buff1, UNS_8 *buff2)
{
	INT_32 idx;

    for (idx = 0; idx < nandgeom.data_bytes_per_page; idx++)
    {
    	if (buff1[idx] != buff2[idx])
    	{
			return idx;
    	}
    }

    return -1;
}

/***********************************************************************
 *
 * Function: compdata
 *
 * Purpose: Verify an entire buffer has the same value
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     buff1 : Pointer to buffer 1 to verify
 *     data : value to verify against
 *
 * Outputs: None
 *
 * Returns: -1 on pass, or failed index
 *
 * Notes: None
 *
 **********************************************************************/
INT_32 compdata(UNS_8 *buff1, UNS_8 data)
{
	INT_32 idx;

    for (idx = 0; idx < nandgeom.data_bytes_per_page; idx++)
    {
    	if (buff1[idx] != data)
    	{
	   		return idx;
    	}
    }

    return -1;
}

/***********************************************************************
 *
 * Function: nand_write_sectors
 *
 * Purpose: Write data to a Small block device
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     sector : Sector to write
 *     wrbuffer : Pointer to buffer to write
 *     size : Total number of bytes
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
UNS_32 nand_write_sectors(int sector, UNS_8 *wrbuffer, int size)
{
	INT_32 bytes = 0;

	while (size > 0)
	{
		bytes = slcnand_write_sector(nand, sector, wrbuffer, NULL);
		if(bytes < 0)
			return 0;
		wrbuffer = wrbuffer + nandgeom.data_bytes_per_page;
		size = size - nandgeom.data_bytes_per_page;
		sector++;
	}
	return 1;
}

/***********************************************************************
 *
 * Function: verify_data
 *
 * Purpose: Verify data written to a device
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     sector : Sector to verify
 *     rdbuffer : Pointer to buffer to verify
 *     size : Total number of bytes
 *
 * Outputs: None
 *
 * Returns: -1 on pass, or failed index
 *
 * Notes: None
 *
 **********************************************************************/
int verify_data(int sector, UNS_8 *wrbuffer, UNS_8 *rdbuffer, int size)
{
	int idx = 0;
	INT_32 bytes = 0;
	while (size > 0)
	{
		bytes = slcnand_read_sector(nand, sector, rdbuffer, NULL);
		if(bytes < 0)
			return 0;

		idx = isblksame(rdbuffer, wrbuffer);
		if (idx >= 0)
  		{
			uart_output((UNS_8 *)"Verify error.\r\n");
   			return idx;
   		}
 		wrbuffer = wrbuffer + nandgeom.data_bytes_per_page;
   		size = size - nandgeom.data_bytes_per_page;
   		sector++;
	}

	return -1;
}

/***********************************************************************
 *
 * Function: c_entry
 *
 * Purpose: Application entry point from the startup code
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Always returns 1, or <0 on an error
 *
 * Notes: None
 *
 **********************************************************************/
void c_entry(void)
{
	UNS_32 noBlocks, sector, progblk, blkcnt;
	UNS_32 loadsize;
	UNS_8 *p8;
	UNS_32 good_blks[64];
	int idx = 0;
	UNS_32 blksize = 0, size = 0, offset = 0, ret;
#ifndef JTAG_LOADING
	UNS_8 tmp[16];
	UNS_32 loadcount, *a1;
#endif

	/* Set virtual address of MMU table */
	cp15_set_vmmu_addr((void *)
		(IRAM_BASE + (256 * 1024) - (16 * 1024)));

	/* Initialize interrupt system */
	int_initialize(0xFFFFFFFF);

	/* Install standard IRQ dispatcher at ARM IRQ vector */
	int_install_arm_vec_handler(IRQ_VEC, (PFV) lpc32xx_irq_handler);

	enable_irq();

    uart_output_init();

#ifndef JTAG_LOADING
	/* Now to download the image */
	uart_output((UNS_8 *)"X");

	/* Wait for 'p' */
	while (!uart_input(&tmp[0], 1));

	/* Wait for 8 bytes from other side */
	idx = 0;
	while (idx < 8)
	{
		idx += uart_input(&tmp[idx], (8 - idx));
	}
	uart_output((UNS_8 *)"o");

	a1 = (UNS_32 *) tmp;

	/* Get size of secondary file */
	loadsize = a1[1];

	/* Receive complete file using UART */
	loadcount = 0;
	p8 = (UNS_8 *) BURNER_LOAD_ADDR;
	while (loadsize > loadcount)
	{
		loadcount += uart_input(p8 + loadcount, (loadsize - loadcount));
	}
	uart_output((UNS_8 *)"t");
#else

    // Fill loading area with 0xCC's.
    memset((void *)STAGE1_LOAD_ADDR, 0xCC, STAGE1_LOAD_SIZE);

	/* Now to download the image */
    uart_output((UNS_8 *)"Load S1L into memory using JTAG.\r\n");

    // Wait for the end of the stage1 load area to be written to
    while (1) {
		volatile UNS_32 temp;
        temp = *((volatile UNS_32 *)(STAGE1_LOAD_ADDR));
        if (temp != 0xCCCCCCCC)
            break;
    }

    uart_output((UNS_8 *)"Calculating S1L size ...\r\n");

    // Look for a word that is not 0xCCCCCCCC
    for (idx=STAGE1_LOAD_SIZE-8; idx>=0; idx-=4) {
		volatile UNS_32 temp;
        temp = *((volatile UNS_32 *)(STAGE1_LOAD_ADDR+idx));
        if (temp != 0xCCCCCCCC)
            break;
    }

    /* Get size of secondary file */
	loadsize = idx;

    uart_output((UNS_8 *)"Burning S1L ... \r\n");
    p8 = (UNS_8 *) STAGE1_LOAD_ADDR;
#endif

	/* Init NAND controller */
	if ((nand = slcnand_board_init()) == 0)
	{
   		uart_output((UNS_8 *)"Cannot initialize SLC NAND device\r\n");
		while (1);
	}

    /* Disable write protect */
    nand_flash_wp_disable();

	slcnand_get_geom(nand, &nandgeom);
	/* Calculate Required Blocks for flashing stage1 application */
	blksize = nandgeom.data_bytes_per_page * nandgeom.pages_per_block;
	noBlocks = loadsize /blksize;
	if ((noBlocks * blksize) < loadsize)
	{
		noBlocks++;
	}

	/* Erase blocks starting from 1 & leave bad blocks */
	progblk = 1;
	uart_output((UNS_8 *)"Formatting blocks...\r\n");
	blkcnt = 0;

	do
	{
		/* Check if block is bad or not */
		ret = slcnand_is_block_bad(nand, progblk);
		if(ret == 0)
		{
			/* Get sector address for first page of block */
			sector = nandgeom.pages_per_block * progblk;
			if (slcnand_erase_block(nand, progblk) != 0)
			{
				/* Erase failure */
				uart_output((UNS_8 *)"Error: Erase failure\r\n");
				while(1);
			}

			/* Really, Block is ereased? */
			slcnand_read_sector(nand, sector, rdbuff, NULL);
			idx = compdata(rdbuff,0xFF);
			if(idx >= 0)
			{
				uart_output((UNS_8 *)"Error: Erase failure...\r\n");
				while (1);
			}

			good_blks[blkcnt] = progblk;
			blkcnt++;
		}
		progblk++;
	}while(blkcnt < noBlocks);

	uart_output((UNS_8 *)"Format complete\r\n");

	uart_output((UNS_8 *)"Writing S1 image into flash...\r\n");
	offset = 0;

	for (blkcnt = 0; blkcnt < noBlocks; blkcnt++)
	{
		progblk = good_blks[blkcnt];
		sector = nandgeom.pages_per_block * progblk;

		if(loadsize <= blksize)
			size = loadsize;
		else
			size = blksize;

		/* Write 1 block size or less data into Flash */
		if (nand_write_sectors(sector, p8 + offset, size) <= 0)
		{
			uart_output((UNS_8 *)"Error: Failed to write data...\r\n");
			while(1);
		}

		idx = verify_data(sector, p8 + offset, rdbuff, size);
		if(idx >= 0)
		{
			uart_output((UNS_8 *)"...Failed\r\n");
			while(1);
		}

		loadsize -= blksize;
		offset += blksize;
		progblk++;
	}
	uart_output((UNS_8 *)"NAND flash is programmed Successfully\r\n");

	/* Loop forever */
	while (1);
}
