/***********************************************************************
 * $Id:: sysapi_flash.c 7837 2011-08-08 06:06:33Z ing03005             $
 *
 * Project: FLASH functions
 *
 * Description:
 *     Implements the following functions required for the S1L API
 *         flash_init
 *         flash_deinit
 *         flash_read_sector
 *         flash_write_sector
 *         flash_erase_block
 *         flash_is_bad_block
 *
 * Note:
 *     These functions need to be developed for the board they will be
 *     executed on. Examples of how these functions work can be seen
 *     in the Phytec and Embedded Artists versions fo S1L. For this
 *     generic package, these functions are stubs.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

#include "sys.h"
#include "s1l_sys.h"
#include "s1l_sys_inf.h"
#include "lpc32xx_slcnand_driver.h"
#include "misc_config.h"
#include "common_funcs.h"

int erasebadblocks = 0;
static INT_32 nand;

/***********************************************************************
 *
 * Function: flash_init
 *
 * Purpose: Initialize NAND device
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns:
 *     Returns pointer to an initialized NAND structure if good, NULL
 *     if bad
 *
 * Notes: None
 *
 **********************************************************************/
NAND_GEOM_T *flash_init(void)
{
#ifdef S1L_SUPPORT_NAND
  static NAND_GEOM_T geom;
  nand_flash_wp_disable();

  sys_up();
  nand = slcnand_board_init();
  if (!nand)
	  return NULL;

  if (slcnand_get_geom(nand, &geom))
	  return NULL;

  return &geom;
#else
	return NULL;
#endif
}

/***********************************************************************
 *
 * Function: flash_deinit
 *
 * Purpose: De-initialize FLASH
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
void flash_deinit(void)
{
#ifdef S1L_SUPPORT_NAND
  slcnand_close(nand);
#endif
}

/***********************************************************************
 *
 * Function: flash_read_sector
 *
 * Purpose: Read a NAND sector, will skip bad blocks
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns:
 *     Returns >0 on success, -1 on fail, or -2 if the block
 *     associated with the sector is bad
 *
 * Notes:
 *     The block is read regardless of whether the block is bad or not.
 *
 **********************************************************************/
int flash_read_sector(UNS_32 sector, void *buff, void *extra)
{
#ifdef S1L_SUPPORT_NAND
	UNS_32 block;
	static UNS_32 lastblock = 0xffffffff;
	static int lastblockgood = 0;
	int ret;

	block = slcnand_get_block(nand, sector);

	if (lastblock != block)
	{
		if (slcnand_is_block_bad(nand, block))
		{
			lastblockgood = 0;
		}
		else
		{
			lastblockgood = 1;
		}

		lastblock = block;
	}

	/* We get 0 on success */
	ret = slcnand_read_sector(nand, sector, (UNS_8 *) buff,
		(UNS_8 *) extra);

	if (lastblockgood == 0)
	{
		return -2;
	}

	/* Caller considers > 0 as success */
	return ret != _ERROR;

#else
	return -1;
#endif
}

/***********************************************************************
 *
 * Function: flash_write_sector
 *
 * Purpose: Writes a NAND sector, will skip bad blocks
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns:
 *     Returns >0 on success, -1 on fail, or -2 if the block
 *     associated with the sector is bad
 *
 * Notes: The block will not be written if it is bad.
 *
 **********************************************************************/
int flash_write_sector(UNS_32 sector, void *buff, void *extra)
{
#ifdef S1L_SUPPORT_NAND
	UNS_32 block;
	static UNS_32 lastblock = 0xffffffff;
	static int lastblockgood = 0;

	block = slcnand_get_block(nand, sector);
	if (lastblock != block)
	{
		if (slcnand_is_block_bad(nand, block))
		{
			lastblockgood = 0;
		}
		else
		{
			lastblockgood = 1;
		}

		lastblock = block;
	}

	if (lastblockgood == 0)
	{
		return -2;
	}

	return slcnand_write_sector(nand, sector, (UNS_8 *) buff,
		(UNS_8 *) extra) != _ERROR;
#else
	return -1;
#endif
}

/***********************************************************************
 *
 * Function: flash_erase_block
 *
 * Purpose: Erase a block
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     block: Block number
 *
 * Outputs: None
 *
 * Returns: TRUE on good erase, otherwise FALSE
 *
 * Notes: None
 *
 **********************************************************************/
BOOL_32 flash_erase_block(UNS_32 block)
{
#ifdef S1L_SUPPORT_NAND
	/* Don't allow erasure of bad blocks in S1L */
	if (!(slcnand_is_block_bad(nand, block)) || erasebadblocks)
	{
		if (slcnand_erase_block(nand, block) != _ERROR)
		{
			return TRUE;
		}
	}
#endif

	return FALSE;
}

/***********************************************************************
 *
 * Function: flash_is_bad_block
 *
 * Purpose: Erase a block
 *
 * Processing:
 *     See function.
 *
 * Parameters:
 *     block: Block number
 *
 * Outputs: None
 *
 * Returns: TRUE if the block is bad, otherwise FALSE
 *
 * Notes: None
 *
 **********************************************************************/
BOOL_32 flash_is_bad_block(UNS_32 block)
{
#ifdef S1L_SUPPORT_NAND
	if (slcnand_is_block_bad(nand, block))
	{
		return TRUE;
	}
#endif

	return FALSE;
}
