/***********************************************************************
 * $Id:: mem_setup.c 3376 2010-05-05 22:28:09Z usb10132                $
 *
 * Project: Memory setup code
 *
 * Description:
 *     Memory interface setup for the board
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only  
 * which provides customers with programming information regarding the  
 * products. This software is supplied "AS IS" without any warranties.  
 * NXP Semiconductors assumes no responsibility or liability for the 
 * use of the software, conveys no license or title under any patent, 
 * copyright, or mask work right to the product. NXP Semiconductors 
 * reserves the right to make changes in the software without 
 * notification. NXP Semiconductors also make no representation or 
 * warranty that such application will be suitable for the specified 
 * use without further testing or modification. 
 **********************************************************************/

#include "startup.h"
#include "dram_configs.h"
#include "lpc32xx_emc.h"
#include "lpc32xx_clkpwr_driver.h"
#include "lpc_arm922t_cp15_driver.h"
#include "board_config.h"

UNS_32 SDRAM_COLS;
#define SDRAM_LP_MASK_BIT (1 << 5)

/***********************************************************************
 *
 * Function: mem_pick_dram
 *
 * Purpose: Setup for 32MB or 64MB DRAM devices
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
static void mem_pick_dram(void)
{
	UNS_32 *p32a, *p32b;

	/* The chip is currently configured for a 32MB device. The 32MB and
	   64MB devices supported on the FDI boards only vary in the number
	   of columns used to access memory. Temporarily switch to the 64MB
	   mapping and run a few memory tests in DRAM. If it fails, we have
	   a 32MB device. */

	/* Try 64M size first */
#if SDRAM_USE_PERFORMANCE_MODE==0
	EMC->emcdynamicconfig0 = (0x11 << 7) | EMC_DYN_DEV_LP_DDR_SDRAM |
		SDRAM_LP_MASK_BIT;
#else
	EMC->emcdynamicconfig0 = (0x11 << 7) | EMC_DYN_DEV_LP_DDR_SDRAM;
#endif

	/* If the MMU is enabled, an uncached region needs to be used for
	   the test. */
	if (cp15_mmu_enabled())
		p32a = (UNS_32 *) 0x90000000;
	else
		p32a = (UNS_32 *) 0x80000000;
	p32b = p32a + ((1 << 10) / 4);

	/* Place a value in the column 10 address with row = 0 and
	   bank = 0 for a 64MB device. If the data written into the column
	   10 address wraps back to the address with column 0, we have a
	   32MB device. */
	*p32a = 0xC00ADDAA;
	*p32b = 0xADD55C10;
	dcache_flush();
	dcache_inval();

	/* If the values in the column 0 and column 10 addresses are the
	   same, the device is a 32MB device */
	if (*p32a == *p32b) {
#if SDRAM_USE_PERFORMANCE_MODE==0
		EMC->emcdynamicconfig0 = (0x0D << 7) | EMC_DYN_DEV_LP_DDR_SDRAM |
			SDRAM_LP_MASK_BIT;
#else
		EMC->emcdynamicconfig0 = (0x0D << 7) | EMC_DYN_DEV_LP_DDR_SDRAM;
#endif
	}
	else
		SDRAM_COLS = 10;
}

/***********************************************************************
 *
 * Function: mem_setup
 *
 * Purpose: Setup memory
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
void mem_setup(void)
{
	UNS_32 dramclk;

    /* Mirror IRAM at address 0x0 */
    CLKPWR->clkpwr_bootmap = CLKPWR_BOOTMAP_SEL_BIT;

	/* Enable HCLK and SDRAM bus clocks before EMC accesses */
	CLKPWR->clkpwr_sdramclk_ctrl = 0;

	/* Enable EMC interface */
	EMC->emccontrol = EMC_DYN_SDRAM_CTRL_EN;
	EMC->emcconfig = 0;

	/* Set a very long dynamic refresh time so the controller isn't
	   always busy refreshing */
	EMC->emcdynamicrefresh = 0x7ff;

	/* Intialized Low Power DDR SDRAM */
	SDRAM_COLS = 9; /* Assume 32M */
	dramclk = clkpwr_get_clock_rate(CLKPWR_SDRAMDDR_CLK);
	ddr_sdram_lp_setup(dramclk);

	/* Do this before enabling port buffers */
	mem_pick_dram();

	/* Enable buffers in AHB ports */
	EMC->emcahn_regs [0].emcahbcontrol = EMC_AHB_PORTBUFF_EN;
	EMC->emcahn_regs [2].emcahbcontrol = EMC_AHB_PORTBUFF_EN;
	EMC->emcahn_regs [3].emcahbcontrol = EMC_AHB_PORTBUFF_EN;
	EMC->emcahn_regs [4].emcahbcontrol = EMC_AHB_PORTBUFF_EN;

	/* Enable port timeouts */
	EMC->emcahn_regs [0].emcahbtimeout = EMC_AHB_SET_TIMEOUT(32);
	EMC->emcahn_regs [2].emcahbtimeout = EMC_AHB_SET_TIMEOUT(32);
	EMC->emcahn_regs [3].emcahbtimeout = EMC_AHB_SET_TIMEOUT(32);
	EMC->emcahn_regs [4].emcahbtimeout = EMC_AHB_SET_TIMEOUT(32);
}
